#include <curl/curl.h>
#include <iostream>
#include <XPath/XPath.hpp>
#include <DOM/SAX2DOM/SAX2DOM.hpp>
#include <SAX/helpers/CatchErrorHandler.hpp>
#include <DOM/io/Stream.hpp>
#include <Taggle/Taggle.hpp>
#include <string>
#include <stdexcept>
#include <sstream>
#include <ciso646>

namespace {
	const char g_testData[] = {
		"<!DOCTYPE html>\n"
		"<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
		"<title>LibReSSL</title>\n"
		"<link rel=\"stylesheet\" href=\"http://www.libressl.org/libre-ssl.css\">\n"
		"<meta charset=\"utf-8\">\n"
		"<meta name=\"resource-type\" content=\"document\">\n"
		"<meta name=\"description\" content=\"the main LibreSSL page\">\n"
		"<meta name=\"keywords\" content=\"LibreSSL,main\"/\n"
		"<meta name=\"distribution\" content=\"global\">\n"
		"</head>\n"
		"<body>\n"
		"<section class=\"libre-ssl\">\n"
		"<hgroup>\n"
		"<header>\n"
		"<h1>LibReSSL</h1>\n"
		"</header>\n"
		"</hgroup>\n"
		"<section class=\"introduction\">\n"
		"<p>LibReSSL is a <strong>FREE</strong> version of the SSL/TLS protocol\n"
		"forked from <a\n"
		"href=\"http://www.youtube.com/watch?v=mRCGDUsdRDU\">OpenSSL</a></p>\n"
		"<p>\n"
		"At the moment we are <a\n"
		"href=\"http://freshbsd.org/search?project=openbsd&q=file.name:libssl\">too\n"
		"busy deleting and rewriting code</a> to make a decent web page. No we\n"
		"don't want help making web pages, thank you.\n"
		"</p>\n"
		"<p>Check back here soon for updates.</p>\n"
		"</section>\n"
		"<section class=\"resources\">\n"
		"<hgroup>\n"
		"<header>\n"
		"<h2>Resources:</h2>\n"
		"</header>\n"
		"</hgroup>\n"
		"<section class=\"open-bsd\">\n"
		"<hgroup>\n"
		"<header>\n"
		"<h3>For OpenBSD</h3>\n"
		"</header>\n"
		"</hgroup>\n"
		"<ul class=\"indent-list\">\n"
		"<li><a href=\"http://www.openbsd.org/ftp.html\">FTP</a></li>\n"
		"<li><a href=\"http://www.openbsd.org/anoncvs.html\">AnonCVS</a></li>\n"
		"<li><a href=\"http://www.openbsd.org/cgi-bin/cvsweb/src/lib/libssl\">CVSWeb</a></li>\n"
		"</ul>\n"
		"</section>\n"
		"<section>\n"
		"<hgroup>\n"
		"<header>\n"
		"<h3>For other OS's</h3>\n"
		"</header>\n"
		"</hgroup>\n"
		"<p>\n"
		"<ul class=\"indent-list\">\n"
		"<li><a href=\"http://www.openbsd.org/papers/bsdcan14-libressl\">LibReSSL Presentation by Bob Beck from BSDCAN 2014</a></li>\n"
		"</ul>\n"
		"</p>\n"
		"<p>\n"
		"Multi OS support will happen once we have\n"
		"</p>\n"
		"<ul>\n"
		"<li><a href=\"http://en.wikipedia.org/wiki/Flensing\">Flensed</a>,\n"
		"refactored, rewritten, and fixed enough of the code so we have stable\n"
		"baseline that we trust and can be maintained/improved.</li>\n"
		"<li>The right Portability team in place.</li>\n"
		"<li>A <a href=\"http://www.openbsdfoundation.org/\">Stable Commitment of\n"
		"Funding</a> to support an increased development and porting\n"
		"effort. </li>\n"
		"</ul>\n"
		"<p>\n"
		"We know you all want this tomorrow. We are working as fast as we can\n"
		"but our primary focus is good software that we trust to run\n"
		"ourselves. <em>We don't want to break your heart</em>.\n"
		"</p>\n"
		"</section>\n"
		"</section>\n"
		"<section class=\"acknowledgements\">\n"
		"<p>\n"
		"LibReSSL is primarily developed by <a\n"
		"href=\"http://www.openbsd.org/\">the OpenBSD Project</a>, and its first\n"
		"inclusion into an operating system will be in <a\n"
		"href=\"http://www.openbsd.org/56.html\">OpenBSD 5.6</a>.\n"
		"</p>\n"
		"<p>\n"
		"LibReSSL is supported financially by <a\n"
		"href=\"http://www.openbsdfoundation.org/\">The OpenBSD Foundation</a> as\n"
		"well as by the <a href=\"http://www.openbsd.org/donations.html\">The\n"
		"OpenBSD Project</a>. Please consider <a\n"
		"href=\"http://www.openbsdfoundation.org/donations.html\">donating</a> to\n"
		"support our efforts.<br>\n"
		"</p>\n"
		"</section>\n"
		"<section class=\"we-love-web-devs-especially-those-that-write-blink-tags-for-us\">\n"
		"<p>This page scientifically designed to annoy web hipsters. <a\n"
		"href=\"http://www.openbsdfoundation.org/donations.html\">Donate now</a>\n"
		"to stop the Comic Sans and Blink Tags</p>\n"
		"</section>\n"
		"</section>\n"
		"</body></html>\n"
	};
} //unnamed namespace

int main (int argc, char* argv[]) {
	//std::cout << g_testData << std::endl;

	Arabica::XPath::XPath<std::string> xpathParser;
	Arabica::XPath::XPathExpressionPtr<std::string> xpath;

	try {
		xpath = xpathParser.compile_expr(argv[1]);
	}
	catch (const std::runtime_error& e) {
		std::cerr << "XPath compilation error: " << e.what() << std::endl;
		return 1;
	}

	typedef Arabica::SAX::Taggle<std::string> TaggleType;
	Arabica::SAX2DOM::Parser<std::string, Arabica::default_string_adaptor<std::string>, TaggleType> domParser;
	Arabica::SAX::CatchErrorHandler<std::string> errorHandler;
	domParser.setErrorHandler(errorHandler);
	Arabica::SAX::InputSource<std::string> inpSource;

	inpSource.setSystemId("deleme_fixed.htm");
	//std::istringstream iss(g_testData);
	//inpSource.setByteStream(iss);
	domParser.parse(inpSource);

	if (not errorHandler.errorsReported()) {
		Arabica::DOM::Document<std::string> doc = domParser.getDocument();
		Arabica::XPath::XPathValuePtr<std::string> result;
		result = xpath->evaluate(doc);
		switch (result->type()) {
		case Arabica::XPath::NODE_SET:
			std::cout << "Result type is a NodeSet\n";
			{
				const Arabica::XPath::NodeSet<std::string>& nodeSet = result->asNodeSet();
				std::cout << "NodeSet size = " << nodeSet.size() << "\n";
				for (unsigned int z = 0; z < nodeSet.size(); ++z) {
					Arabica::DOM::Node<std::string> node = nodeSet[z];
					std::cout << node << std::endl;
				}
			}
			break;

		default:
			std::cout << "Result type is a String\n";
			std::cout << result->asString() << std::endl;
		}
	}
	else {
		std::cerr << errorHandler.errors() << std::endl;
		errorHandler.reset();
	}
	return 0;
}
